/*
 *
 * Authors: Christos Bakogiannis (mpakoyan "at" gmail "dot" com)
 *          Nikolaos Karapanos   (karapas "at" gmail "dot" com)
 *
 *
 * This file is part of kmGNFS.
 *
 * This distribution of the General Number Field Sieve algorithm is
 * placed in the public domain by its authors. It is distributed with
 * the hope that it will be useful, and you can use it freely for any
 * purpose without having to notify anyone. However, if you find our
 * work useful in any way, please let us know about it.
 *
 * kmGNFS makes use of Victor Shoup's NTL free library.
 * NTL can be used in conjunction with GMP (the GNU Multi-Precision library)
 * for enhanced performance. For more info, visit NTL's home page.
 *
 * We disclaim any responsibility for any bug.
 *
 */


#include <cstdlib>
#include <sstream>
#include <fstream>

#include <NTL/ZZ.h>

NTL_CLIENT

int main(int argc, char** argv) {

    ZZ prime_1, prime_2, product;
    long numOfDigits, numOfDigitsInEachPrime, numOfBitsInEachPrime, digitsOfProduct;
    const char* productFile = "gnfs_ProductToTest.txt";

    ofstream f(productFile);

    if (!f.is_open()) {
        cout << "Error opening " << productFile << ". Exiting..." << endl;
        exit(EXIT_FAILURE);
    }


    cout << "How many digits do you want the product of the two primes to have?" << endl;
    cout << "> ";

    cin >> numOfDigits;

    numOfDigitsInEachPrime = numOfDigits / 2;
    numOfBitsInEachPrime = numOfDigitsInEachPrime * 3;

    while (digitsOfProduct != numOfDigits) {

        prime_1 = GenPrime_ZZ(numOfBitsInEachPrime);
        prime_2 = GenPrime_ZZ(numOfBitsInEachPrime);

        mul(product, prime_1, prime_2);

        digitsOfProduct = (log(product) / log(10)) + 1;

        //cout << "digitsOfProduct -> " << digitsOfProduct << endl;

        numOfBitsInEachPrime++;
    }

    cout << "The product is   -> " << product << endl;
    cout << "The 1st prime is -> " << prime_1 << endl;
    cout << "The 2nd prime is -> " << prime_2 << endl;

    f << "Generating a " << numOfDigits << " digit number as a product of two primes..." << endl << endl;
    f << "The product is   -> " << product << endl;
    f << "The 1st prime is -> " << prime_1 << endl;
    f << "The 2nd prime is -> " << prime_2 << endl;





    f.close();

    cout << endl;

    return (EXIT_SUCCESS);
}

